
/*
RC Light Switch Controller

The code for the remote control light switch.

Alan Merritt
12/13/2021
*/

//Include some libraries for the RF module and the servo.
#include <SPI.h>
#include <RF24.h>
#include <Servo.h>

//Declare our RF24 radio. Pin number 9 on the Arduino
//connects to the CE pin on the module and pin 10 connects
//to its CSN pin.
RF24 radio(9, 10);

//This is the address with which the two modules will use to communicate.
//Make sure that the address that the controller uses is the same.
byte address[] = "00001";

//The pin that the front pushbutton is connected to.
const int buttonPin = 7;

//!!! These control how far the servo turns. !!!
//You may need to adjust these for your switch.
//The motor position for when the lights are turned on.
const int upPosition = 40;
//The motor position for when the lights are turned off.
const int downPosition = 150;

//This variable states whether the light should be on or off.
//true is on and false is off.
bool currentState = false;

//This variable is set to true if either the front pushbutton
//is pushed or the controller sends a signal.
bool controlTriggered = false;
//This variable is the value of the controlTriggered variable
//the previous time the program looped. If controlTriggered is
//true and controlTriggeredLast is false, we know to switch the switch.
bool controlTriggeredLast = false;

//The servo motor.
Servo switcher;

void setup() {

  Serial.begin(9600);  
  
  //Set up the RF radio in read mode.
  radio.begin();

  radio.setPALevel(RF24_PA_LOW);
  
  radio.openReadingPipe(0, address);

  radio.startListening();

  Serial.println("Radio started.");

  //Set the button pin to an input mode. The button is
  //connected to the ground pin and a digital pin. The
  //INPUT_PULLUP mode makes it so that we don't need a resistor.
  pinMode(buttonPin, INPUT_PULLUP);

  //Setup the servo.
  switcher.attach(6);
  //Set the servo to its downward position when it starts up.
  switcher.write(downPosition);

  Serial.println("Ready.");

}

void loop() {
  
  //A variable to hold the data recieved by the radioo.
  bool recieved = false;

  //Get the data (if there is any).
  while(radio.available())
  {
    //This line plugs the data we get into the variable 'recieved'.
    radio.read(&recieved, sizeof(bool));
    Serial.println(recieved);
  }

  //Determine if the button on the device or the one on the controller has been pressed.
  controlTriggered = !digitalRead(buttonPin) || recieved;

  //If it has been pressed, but it was not pressed on the previous loop,
  //it means that it has just been pressed (as opposed to held) and 
  //that the light should be switched.
  if(controlTriggered && !controlTriggeredLast)
  {
    //Change the state to the opposite of what it is now.
    currentState = !currentState;
    Serial.println("Switching.");
  }

  //Move the servo based on the state (true = up, false = down).
  if(currentState == true)
  {
    switcher.write(upPosition);
  } else {
    switcher.write(downPosition);
  }
  
  //Update the controlTriggeredLast variable so that on the next loop
  //it will have the value that controlTriggered had on this loop.
  controlTriggeredLast = controlTriggered;

  delay(5);

}
